@@ -1,3 +1,6 @@
+//go:build !remote
+// +build !remote
+
 package libimage
 
 import (
@@ -113,8 +116,8 @@ func TestFilterDigest(t *testing.T) {
 		matches int
 		id      string
 	}{
-		{string(busybox.Digest()), 1, busybox.ID()},
-		{string(alpine.Digest()), 1, alpine.ID()},
+		{string(busybox.Digest()[:10]), 1, busybox.ID()},
+		{string(alpine.Digest()[:10]), 1, alpine.ID()},
 	} {
 		listOptions := &ListImagesOptions{
 			Filters: []string{"digest=" + test.filter},
@@ -130,6 +133,45 @@ func TestFilterDigest(t *testing.T) {
 	assert.Error(t, err)
 }
 
+func TestFilterID(t *testing.T) {
+	busyboxLatest := "quay.io/libpod/busybox:latest"
+	alpineLatest := "quay.io/libpod/alpine:latest"
+
+	runtime, cleanup := testNewRuntime(t)
+	defer cleanup()
+	ctx := context.Background()
+
+	pullOptions := &PullOptions{}
+	pullOptions.Writer = os.Stdout
+
+	pulledImages, err := runtime.Pull(ctx, busyboxLatest, config.PullPolicyMissing, pullOptions)
+	require.NoError(t, err)
+	require.Len(t, pulledImages, 1)
+	busybox := pulledImages[0]
+
+	pulledImages, err = runtime.Pull(ctx, alpineLatest, config.PullPolicyMissing, pullOptions)
+	require.NoError(t, err)
+	require.Len(t, pulledImages, 1)
+	alpine := pulledImages[0]
+
+	for _, test := range []struct {
+		filter  string
+		matches int
+		id      string
+	}{
+		{busybox.ID()[:5], 1, busybox.ID()},
+		{alpine.ID()[:5], 1, alpine.ID()},
+	} {
+		listOptions := &ListImagesOptions{
+			Filters: []string{"id=" + test.filter},
+		}
+		listedImages, err := runtime.ListImages(ctx, nil, listOptions)
+		require.NoError(t, err, "%v", test)
+		require.Len(t, listedImages, test.matches, "%s -> %v", test.filter, listedImages)
+		require.Equal(t, listedImages[0].ID(), test.id)
+	}
+}
+
 func TestFilterManifest(t *testing.T) {
 	busyboxLatest := "quay.io/libpod/busybox:latest"
 	alpineLatest := "quay.io/libpod/alpine:latest"
